#pragma rtGlobals=1		// Use modern global access method.
#pragma ModuleName=specImage
#pragma version = 0.10
#pragma IgorVersion = 6.2
#include "spec", version>=2.23
#include "Diffractometer", version >=0.15
#include "ImageDisplayScaling", version >=1.84
#include "QspaceVolumesView", version >=0.99


Menu "Qspace"
	"Make Qspace Volume from spec Images...",MakeQspaceVolumeSpec("")
	help = {"Make and Fill a 3D Qspace array from a spec scan with a 2D detector."}
	"-"
End

Menu "Data"
	Submenu "spec"
		"Load Pilatuf spec Image...",LoadSpecImage(NaN,NaN)
	End
End

Menu "Load Waves"
	"Load Pilatuf spec Image...",LoadSpecImage(NaN,NaN)
End

Menu "Diffractometer"
	SubMenu "spec Images"
		"Load spec Image...",LoadSpecImage(NaN,NaN)
		help = {"Load one image from a spec scan with 2D detector."}
		"Re Plot spec Image...", rePlotDiffractometerImage(NaN,NaN)
		help = {"Re-Plot one image from a diffractometer scan."}
	End
	"-"
End


//  ============================================================================  //
//  ============================= Start of Qspace Histogram ============================  //

Function/WAVE MakeQspaceVolumeSpec(scanRange,[mask])
	String scanRange						// combine all images from every point in each scan from scanRange
	Wave mask								// used pixels that are 1, ignore pixels that are 0

	String motors = StrVarOrDefault("root:Packages:Diffractometer:DiffractometerAxisNames","")
	Variable Nm=ItemsInList(motors)
	STRUCT detectorGeometry d
	if (FillDetectorStructDefault(d))				//fill the detector structure with current values
		DoAlert 0, "No Detector Structure, please set one"
		return $""
	elseif (Nm<1)
		DoAlert 0, "Unable to find list of motor names, set Diffractometer Type"
		return $""
	endif
	if (ItemsInRange(scanRange)<1)				// bad or empty range, prompt user
		String maskName=NameOfWave(mask), maskList=WaveListClass("imageMask","*","DIMS:2")
		Prompt maskName,"mask to use with all images",popup, "_none_;"+maskList
		Prompt scanRange,"range of spec scans to process together, e.g. \"1-20\""
		if (ItemsInList(maskList)>0)
			DoPrompt "scans",scanRange,maskName
		else
			DoPrompt "scans",scanRange
		endif
		if (V_flag)
			return $""
		endif
		maskName = SelectString(stringmatch(maskName,"_none_"),maskName,"")
		Wave mask = $maskName
		printf "MakeQspaceVolumeSpec(\"%s\"", scanRange
		if (strlen(maskName))
			printf ", mask=%s",maskName
		endif
		printf ")\r"
	endif
	if (ItemsInRange(scanRange)<1)				// number of scans
		return $""
	endif

	STRUCT boundingVolume v
	initBoundingVolumeStruct(v)

	Make/N=(Nm)/D/FREE A
	Make/N=3/D/FREE q0,q1
	String wnote, str
	Variable m, kev, val
	Variable scanNum,N								// N counts number of images to process
	Variable trans,I0, AvgI0=0,NAvgI0=0, scale
	Variable ip,Np									// for looping over points in one scan
	for (N=0,scanNum=str2num(scanRange); numtype(scanNum)==0; scanNum=NextInRange(scanRange,scanNum))
		for (m=0;m<Nm;m+=1)
			A[m] = specInfo(scanNum,StringFromList(m,motors))
		endfor
		kev = specInfo(scanNum,"DCM_energy")
		Np = specInfo(scanNum,"Npoints")			// number of points in this scan
		for (ip=0;ip<Np;ip+=1)					// for each point in this scan
			Wave image = $LoadSpecImage(scanNum,ip,extras="quiet:1")
			if (!WaveExists(image))
				sprintf str,"Could not load scan %g,  point %g",scanNum,ip
				print str
				DoAlert 0,str
				return $""
			endif
			wnote = note(image)
			KillWaves/Z image

			for (m=0;m<Nm;m+=1)
				val = NumberByKey(StringFromList(m,motors),wnote,"=")
				A[m] = numtype(val) ? A[m] : val
			endfor
			val = NumberByKey("keV",wnote,"=")	;	keV = numtype(val) ? keV : val
			diffractometer#QofPixel(d,keV,d.Nx/2,d.Ny/2,A,q1)
			if (N==0)
				q0 = q1
			endif
			extendQvolumeFromImage(d,keV,A,v)

			I0 = NumberByKey("I0",wnote,"=")
			if (I0>0)
				AvgI0 += I0	
				NAvgI0 += 1
			endif
			N+=1
		endfor
	endfor
	AvgI0 = (numtype(AvgI0)==0 || AvgI0>0 || NAvgI0>0) ? AvgI0/NAvgI0 : 1
	printf "Found %g images to process,  with <I0> = %g\r",N,AvgI0
	print "Measured Q-Volume is: ",boundingVolumeStruct2str(v)

	String QspaceNameEnd=num2str(str2num(scanRange))
	QspaceNameEnd += SelectString(ItemsInRange(scanRange)>1,"","_"+num2str(lastInRange(scanRange)))
	String QspaceName="Qspace"+QspaceNameEnd
	Make/N=3/D/FREE q01 = q1-q0				// find dQ
	Variable dQ = norm(q01)/(N-1)
print "dQ start = ",dQ
	dQ = max(dQ,max(max(v.dx,v.dy),v.dz)/100)
print "dQ after = ",dQ
	Variable Nx=ceil(v.dx / dQ)+1, Ny=ceil(v.dy / dQ)+1, Nz=ceil(v.dz / dQ)+1

	Variable Nqtot = Nx*Ny*Nz
	Make/N=(Nx,Ny,Nz)/D/O $QspaceName=0
	Wave Qspace=$QspaceName
	Make/N=(Nx,Ny,Nz)/I/FREE QspaceNorm=0
	SetScale/P x v.xlo,dQ,"nm\S-1\M", Qspace, QspaceNorm
	SetScale/P y v.ylo,dQ,"nm\S-1\M", Qspace, QspaceNorm
	SetScale/P z v.zlo,dQ,"nm\S-1\M", Qspace, QspaceNorm
	v.xhi = (v.xlo) + (Nx-1)*dQ
	v.yhi = (v.ylo) + (Ny-1)*dQ
	v.zhi = (v.zlo) + (Nz-1)*dQ
	updateBoundingVolumeStruct(v)
	print "Histogram Q-Volume is: ",boundingVolumeStruct2str(v)
	printf "dQ = %g 1/nm,   N = [%g, %g, %g],   %g points\r",dQ,Nx,Ny,Nz,Nqtot
	Variable unWrapped=0							// flags that unwrapping occured

	String progressWin = ProgressPanelStart("",stop=1,showTime=1)
	Variable i, stop=0
	for (i=0, scanNum=str2num(scanRange); numtype(scanNum)==0 && stop==0; scanNum=NextInRange(scanRange,scanNum))
		for (m=0;m<Nm;m+=1)
			A[m] = specInfo(scanNum,StringFromList(m,motors))
		endfor
		kev = specInfo(scanNum,"DCM_energy")
		Np = specInfo(scanNum,"Npoints")			// number of points in this scan
		for (ip=0;ip<Np;ip+=1)					// for each point in this scan
			if (ProgressPanelUpdate(progressWin,i/N*100))
				stop = 1
				break
			endif
			Wave image = $LoadSpecImage(scanNum,ip,extras="quiet:1")
			if (!WaveExists(image))
				sprintf str,"Could not load scan %g,  point %g",scanNum,ip
				print str
				DoAlert 0,str
				return $""
			endif
			wnote = note(image)
			for (m=0;m<Nm;m+=1)
				val = NumberByKey(StringFromList(m,motors),wnote,"=")
				A[m] = numtype(val) ? A[m] : val
			endfor
			val = NumberByKey("keV",wnote,"=")	;	keV = numtype(val) ? keV : val
			trans = NumberByKey("trans",wnote,"=")
			trans = trans>0 ? trans : 1
			I0 = NumberByKey("I0",wnote,"=")
			I0 = I0 > 0 ? I0 : 1
			scale = AvgI0 / (trans*I0)				// correction for filters and I0

			#ifdef EnablePilatusUnWrapping		// to enable the unwrappng functions
				unWrapped = UnWrapImage(image) || unWrapped
			#endif

			HistogramOneImage(d,A,kev,image,Qspace, QspaceNorm,scale,mask)
			KillWaves/Z image
			i += 1
		endfor
	endfor
	printf "  the main part took = %s\r",Secs2Time(SecondsInProgressPanel(progressWin),5,0)
	DoWindow/K $progressWin

	MatrixOP/FREE eq0 = equal(QspaceNorm,0)		// set to 1 if point not used
	Variable zeros=sum(eq0)						// number of zero points in Qspace

	Duplicate/FREE QspaceNorm,QspaceNorm1
	QspaceNorm1 = max(QspaceNorm,1)			// avoid divide by zero
	Qspace /= QspaceNorm1							// normalize by number of times each point sampled
	wnote = ReplaceStringByKey("waveClass",wnote,"Qspace3D","=")
	for (m=0;m<Nm;m+=1)
		wnote = RemoveByKey(StringFromList(m,motors),wnote,"=")
	endfor
	wnote = ReplaceNumberByKey("zeros", wnote,zeros,"=")
	Wave qvec = BraggPeak(Qspace)
	wnote = ReplaceStringByKey("qPeak", wnote,vec2str(qvec,bare=1),"=")
	Note/K Qspace, wnote
	Note/K QspaceNorm, ReplaceStringByKey("waveClass", wnote,"Qspace3DNorm","=")

	WaveStats/Q/M=1 Qspace
	printf "V_npnts = %g, V_numNaNs = %g, V_numINFs = %g\r",V_npnts, V_numNaNs, V_numINFs
	printf "V_avg = %g,  V_Sum = %g,  V_sdev = %g\r",V_avg, V_Sum, V_sdev
	printf "there are %g zeros, and %g non-zero points  (total = %g)\r",zeros,Nqtot-zeros,Nqtot
	printf "min[%g, %g, %g] = %g\r",V_minRowLoc,V_minColLoc,V_minLayerLoc,V_min
	printf "max[%g, %g, %g] = %g\r",V_maxRowLoc,V_maxColLoc,V_maxLayerLoc,V_max
	#ifdef EnablePilatusUnWrapping
		printf "%s\r", SelectString(unWrapped,"NO Unwrapping needed","Images were UNwrapped")
	#endif
	MakeSampledPoints(QspaceNorm,name="SampledPoints"+QspaceNameEnd+"XYZ")
	MakeGizmocubeCorners(Qspace)
	Qspace = Qspace==0 ? NaN : Qspace

	return Qspace
End
//
Static Function HistogramOneImage(d,A,keV,image,Qspace, QspaceNorm,scale,mask)
	STRUCT detectorGeometry &d
	Wave A
	Variable keV
	Wave image
	Wave Qspace, QspaceNorm
	Variable scale
	Wave mask
	scale = numtype(scale) || scale==0 ? 1 : scale
	if (!WaveExists(mask))
		Make/N=(2,2)/FREE mask=1			// this forces mask[i][j] to always be 1 
	endif

	Make/N=3/D/FREE qvec
	Variable dx=DimDelta(Qspace,0), dy=DimDelta(Qspace,1), dz=DimDelta(Qspace,2)
	Variable x0=DimOffset(Qspace,0), y0=DimOffset(Qspace,1), z0=DimOffset(Qspace,2)
	Variable Nx = d.Nx, Ny = d.Ny
	Variable i,j, mx,my,mz
	for (j=0;j<Ny;j+=1)
		for (i=0;i<Nx;i+=1)
			if (mask[i][j])
				diffractometer#QofPixel(d,keV,i,j,A,qvec)
				mx = round((qvec[0]-x0)/dx)
				my = round((qvec[1]-y0)/dy)
				mz = round((qvec[2]-z0)/dz)
				Qspace[mx][my][mz] += image[i][j] * scale
				QspaceNorm[mx][my][mz] += 1
			endif
		endfor
	endfor
End
//
Static Function extendQvolumeFromImage(d,keV,A,v)
	STRUCT detectorGeometry &d
	Variable keV
	Wave A
	STRUCT boundingVolume &v

	Variable Nx = d.Nx, Ny = d.Ny
	Make/N=3/D/FREE qvec

	diffractometer#QofPixel(d,keV,0,0,A,qvec)						// (0,0) corner of detector
	extendBoundingVolumeStruct(v,qvec)

	diffractometer#QofPixel(d,keV,Nx-1,0,A,qvec)					// (Xmax,0) corner of detector
	extendBoundingVolumeStruct(v,qvec)

	diffractometer#QofPixel(d,keV,0,Ny-1,A,qvec)					// (0,Ymax) corner of detector
	extendBoundingVolumeStruct(v,qvec)

	diffractometer#QofPixel(d,keV,Nx-1,Ny-1,A,qvec)				// (Xmax,Ymax) corner of detector
	extendBoundingVolumeStruct(v,qvec)
End


//Function/S Rescale3DbyQ4(Qspace)
//	Wave Qspace
//
//	Variable qx,qy,qz
//	sscanf StringByKey("qpeak", note(Qspace),"="), "%g, %g, %g", qx,qy,qz
//	if (V_flag!=3)
//		return ""
//	endif
//
//	Variable dx=DimDelta(Qspace,0), dy=DimDelta(Qspace,1), dz=DimDelta(Qspace,2)
//	Variable x0=DimOffset(Qspace,0), y0=DimOffset(Qspace,1), z0=DimOffset(Qspace,2)
//	Duplicate/FREE Qspace, Q4
//	Q4 = Qspace[p][q][r] * ( (p*dx+x0-qx)^2 + (q*dy+y0-qy)^2 + (r*dz+z0-qz)^2) ^2
//	String wName = NameOfWave(Qspace)+"Q4"
//	Duplicate/O Q4, $wName
//	Note/K $wName,ReplaceNumberByKey("Qpower",note(Qspace),4,"=")
//	printf "rescaled '%s' by q^4, using a Bragg peak at {%g, %g, %g}(1/nm), result in '%s'\r",NameOfWave(Qspace),qx,qy,qz,wName
//	return wName
//End


Static Function MakeSampledPoints(Qspace,[name])
	Wave Qspace
	String name
	name = SelectString(ParamIsDefault(name),name,"SampledPointsXYZ")
	name = SelectString(strlen(name),"SampledPointsXYZ",name)

	Variable Nx=DimSize(Qspace,0), Ny=DimSize(Qspace,1), Nz=DimSize(Qspace,2)
	Variable N=Nx*Ny*Nz

	Variable dx=DimDelta(Qspace,0), dy=DimDelta(Qspace,1), dz=DimDelta(Qspace,2)
	Variable x0=DimOffset(Qspace,0), y0=DimOffset(Qspace,1), z0=DimOffset(Qspace,2)
	Variable xx,yy,zz

	Make/N=(N,3)/O $name=NaN
	Wave sampledXYZ = $name

	Variable i,j,k, m
	for (k=0,m=0;k<Nz;k+=1)
		zz = k*dz + z0
		for (j=0;j<Ny;j+=1)
			yy = j*dy + y0
			for (i=0;i<Nx;i+=1)
				xx = i*dx + x0
				if (Qspace[i][j][k]>0)
					sampledXYZ[m][0] = xx
					sampledXYZ[m][1] = yy
					sampledXYZ[m][2] = zz
					m += 1
				endif
			endfor
		endfor
	endfor
	Redimension/N=(m,-1) sampledXYZ

	String wnote="waveClass=sampledPoints;"
	wnote = ReplaceStringByKey("sourceWave",wnote,NameOfWave(Qspace),"=")
	wnote = ReplaceStringByKey("sourceWaveFullPath",wnote,GetWavesDataFolder(Qspace,2),"=")
	Note/K sampledXYZ, wnote
End

//  ============================= End of Qspace Histogram =============================  //
//  ============================================================================  //



//  ============================================================================  //
//  ============================ Start of spec Image Loading ============================  //

Function/T LoadSpecImage(scanNum,pindex[,extras])
	Variable scanNum											// scanNumber in spec file
	Variable pindex												// point in scanNum
	String extras												// example		"quiet:1;multiple:0;"
	extras = SelectString(ParamIsDefault(extras),extras,"")
	Variable quiet = NumberByKey("quiet",extras)				// for quiet=1, if any problems just return
	Variable multiple = NumberByKey("multiple",extras)		// load multiple images, NOT YET WORKING
	quiet = numtype(quiet) ? 0 : !(!quiet)
	multiple = numtype(multiple) ? 0 : !(!multiple)

	if (!(scanNum>0) || !(pindex>=0) || numtype(scanNum+pindex))
		scanNum = scanNum>0 ? scanNum : NumVarOrDefault("root:Packages:spec:lastScan", 1)
		pindex = pindex>=0 ? pindex : 0
		Prompt scanNum,"spec Scan Number"
		Prompt pindex,"point in spec scan"
		DoPrompt "spec Scan & Point", scanNum,pindex
		if (V_flag)
			return ""
		endif

		if (!(scanNum>0) || numtype(scanNum))
			printf "scan number %g is invalid\r",scanNum
		endif
		if (!(pindex>=0) || numtype(pindex))
			printf "point number %g is invalid\r",pindex
		endif
	endif

	String 	specFldr
	sprintf specFldr,"%sspec%d:",StrVarOrDefault("root:Packages:spec:specDataFolder",""),scanNum
	if (!DataFolderExists(specFldr))
		printf "spec scan %g has not been loaded, you must load it first\r",scanNum
		return ""
	endif

	DFREF dfr=$specFldr										// Reference to folder with spec scan
	Wave ww=$(specFldr+GetIndexedObjNameDFR(dfr,1,0))
	if (!WaveExists(ww))
		return ""
	elseif (pindex<0 || pindex>=DimSize(ww,0) || numtype(pindex))
		printf "spec scan %g has been loaded, BUT it has only %d points, you asked for point=%g\r",scanNum,DimSize(ww,0),pindex
		return ""
	endif
	String specPath = StrVarOrDefault("root:Packages:spec:specDefaultFile","")
	String fullName												// full path+name to file
	String name=ParseFilePath(3, specPath,":", 0,0)
	sprintf fullName, "%simages:%s:S%03d:%s_S%03d_%05d.tif", ParseFilePath(1, specPath,":", 1,0),name,scanNum,name,scanNum,pindex
	GetFileFolderInfo/Q/Z=1 fullName							// check if file exists
	PathInfo home
	if (!V_isFile && V_flag)									// look for a path relative to "home" path, if home exists
		String str = ":"+ParseFilePath(0,S_path,":",1,0)+":"
		Variable ii = strsearch(fullName,str,0)
		ii += strlen(str)
		fullName = S_path+fullName[ii,Inf]
		GetFileFolderInfo/P=home/Q/Z=1 fullName				// again, check if file exists
		if (!V_isFile)											// file does not exist, give up looking
			if (!quiet)
				printf "Could not find file  '%s'\r",fullName
			endif
			return ""
		endif
	endif

	String imageFldr,wName								// name of wave to hold image
	sprintf wName,"%s_S%d_%d",name,scanNum,pindex
	wName = CleanupName(wName,0)
	sprintf specFldr,"%sspec%d:",StrVarOrDefault("root:Packages:spec:specDataFolder",""),scanNum
	sprintf imageFldr,"%sspec%d:images:",StrVarOrDefault("root:Packages:spec:specDataFolder",""),scanNum

	String fldrSav= GetDataFolder(1)
	SetDataFolder $specFldr
	NewDataFolder/O/S images
	if (ParamIsDefault(extras))
		Wave image = $LoadGenericImageFile(fullName)
	else
		Wave image = $LoadGenericImageFile(fullName,extras=extras)
	endif
	SetDataFolder fldrSav
	if (!quiet)
		printf "LoadSpecImage(%g,%g",scanNum,pindex
		if (!ParamIsDefault(extras))
			printf ", quiet=%g",quiet
		endif
		if (!ParamIsDefault(extras))
			printf ", multiple=%g",multiple
		endif
		if (WaveExists(image))
			printf ") // wave = '%s'\r",imageFldr+NameOfWave(image)
		else
			printf ") // NO wave loaded\r"
		endif
	endif
	if (!WaveExists(image))
		return ""
	endif

	String wnote=note(image), noteMore=""					// used to put position of this point into wave note
	Variable i,N=CountObjects(specFldr,1)
	if (N>0)
		for (i=0;i<N;i+=1)
			str = GetIndexedObjName(specFldr,1,i)
			if (StringMatch(str,"Epoch"))
				break
			endif
			Wave ww=$(specFldr+str)
			if (WaveExists(ww))
//				filters = stringmatch(str,"filters") ? ww[pindex] : filters
//				trans = stringmatch(str,"trans") ? ww[pindex] : trans
//				I0 = stringmatch(str,"I0") ? ww[pindex] : I0
				noteMore = ReplaceNumberByKey(str,noteMore,ww[pindex],"=")
			endif
		endfor
	endif
	if (i<N)
		wnote += noteMore
	endif
	if (exists(specFldr+"filters"))							// special for filters
		Wave ww=$(specFldr+"filters")
		Variable filters = ww[pindex]
		filters = numtype(filters) ? 0 : filters
		if (filters>=0)
			wnote = ReplaceNumberByKey("filters",wnote,filters,"=")
		endif
	endif
	Variable trans=NaN
	if (exists(specFldr+"trans"))							// special for computed tranmission from filters
		Wave ww=$(specFldr+"trans")
		trans = ww[pindex]
	endif
	trans = numtype(trans) ? specInfo(scanNum,"PF4_trans") : trans
	if (trans>0 && trans<=1)
		wnote = ReplaceNumberByKey("trans",wnote,trans,"=")
	endif
	if (exists(specFldr+"I0"))								// special for I0
		Wave ww=$(specFldr+"I0")
		Variable I0 = ww[pindex]
		if (I0>0 && numtype(I0)==0)
			wnote = ReplaceNumberByKey("I0",wnote,I0,"=")
		endif
	endif

	Note/K image, wnote
	if (!quiet)
		GraphDiffractometerImage(image)
	endif
	return GetWavesDataFolder(image,2)
End


Function/T LoadSpecImageROI(scanNum,pindex,i0,i1,j0,j1,[extras])
	Variable scanNum				// scanNumber in spec file
	Variable pindex					// point in scanNum
	Variable i0,i1,j0,j1			// pixel range of ROI (if i1 or j1<0 then use whole image)
	String extras					// example		"quiet:1;multiple:0;"
	extras = SelectString(ParamIsDefault(extras),extras,"")

	if (ParamIsDefault(extras))
		Wave image = $LoadSpecImage(scanNum,pindex)
		//	wName = LoadPilatusTiffImageSpec(scanNum,pindex)
	else
		Wave image = $LoadSpecImage(scanNum,pindex,extras=extras)
		//	wName = LoadPilatusTiffImageSpec(scanNum,pindex,extras=extras)
	endif
	if (!WaveExists(image))
		return ""
	endif
	String wnote = note(image)

	Variable ydim, xdim									// x,y size of whole array
	xdim = DimSize(image,0)
	ydim = DimSize(image,1)
	i1 = (i1<1) ? xdim-1 : i1							// -1 flags use whole range
	j1 = (j1<1) ? ydim-1 : j1

	i0 = max(round(i0),0)
	i1 = max(round(i1),0)
	j0 = max(round(j0),0)
	j1 = max(round(j1),0)

	Variable nx = i1-i0+1
	Variable ny = j1-j0+1
	if (nx<1 || ny<1)									// nothing to read
		KillWaves/Z image
		return ""
	endif

	Duplicate/R=[i0,i1][j0,j1]/FREE image, ROIwave	// save the ROI I want
	Redimension/N=(nx,ny,-1) image
	image = ROIwave									// re-set image to ROI values
	WAVEClear ROIwave

	wnote = ReplaceNumberByKey("startx",wnote,i0,"=")
	wnote = ReplaceNumberByKey("endx",wnote,i1,"=")
	wnote = ReplaceNumberByKey("starty",wnote,j0,"=")
	wnote = ReplaceNumberByKey("endy",wnote,j1,"=")	
	Note/K image,wnote
	return GetWavesDataFolder(image,2)
End

//  ============================ End of spec Image Loading =============================  //
//  ============================================================================  //



//  ============================================================================  //
//  ============================ Start of spec Image Plotting ============================  //

Function/T rePlotDiffractometerImage(scanNum,pindex,[extras])
	Variable scanNum											// scanNumber in spec file
	Variable pindex												// point in scanNum
	String extras												// example		"quiet:1;multiple:0;"
	extras = SelectString(ParamIsDefault(extras),extras,"")

	if (!(scanNum>0) || !(pindex>=0) || numtype(scanNum+pindex))
		scanNum = scanNum>0 ? scanNum : NumVarOrDefault("root:Packages:spec:lastScan", 1)
		pindex = pindex>=0 ? pindex : 0
		Prompt scanNum,"spec Scan Number"
		Prompt pindex,"point in spec scan"
		DoPrompt "spec Scan & Point", scanNum,pindex
		if (V_flag)
			return ""
		endif
		if (!(scanNum>0) || numtype(scanNum))
			printf "scan number %g is invalid\r",scanNum
		endif
		if (!(pindex>=0) || numtype(pindex))
			printf "point number %g is invalid\r",pindex
		endif
		printf "rePlotDiffractometerImage(%g,%g",scanNum,pindex
		if (!ParamIsDefault(extras))
			printf ", extras=\"%s\"",extras
		endif
		printf ")\r"
	endif
	String 	specFldr
	sprintf specFldr,"%sspec%d:",StrVarOrDefault("root:Packages:spec:specDataFolder",""),scanNum
	if (!DataFolderExists(specFldr))
		printf "spec scan %g has not been loaded, you must load it first\r",scanNum
		return ""
	endif
	DFREF dfr=$specFldr										// Reference to folder with spec scan
	Wave ww=$(specFldr+GetIndexedObjNameDFR(dfr,1,0))
	if (!WaveExists(ww))
		return ""
	elseif (pindex<0 || pindex>=DimSize(ww,0) || numtype(pindex))
		printf "spec scan %g has been loaded, BUT it has only %d points, you asked for point=%g\r",scanNum,DimSize(ww,0),pindex
		return ""
	endif

	String imageFldr,wName								// name of wave to hold image
	String specPath = StrVarOrDefault("root:Packages:spec:specDefaultFile","")
	String name=ParseFilePath(3, specPath,":", 0,0)
	sprintf wName,"%s_S%d_%d",name,scanNum,pindex
	wName = CleanupName(wName,0)
	sprintf specFldr,"%sspec%d:",StrVarOrDefault("root:Packages:spec:specDataFolder",""),scanNum
	sprintf imageFldr,"%sspec%d:images:",StrVarOrDefault("root:Packages:spec:specDataFolder",""),scanNum
	Wave image = $(imageFldr+wName)
	if (WaveExists(image))
		GraphDiffractometerImage(image)
	else
		String str
		sprintf str, "In scan %g, image #%g is not loaded",scanNum,pindex
		printf str
		DoAlert 0, str
	endif
	return ""
End
//
Static Function/T GraphDiffractometerImage(image)
	Wave image
	if (!WaveExists(image))
		return ""
	endif
	String win=StringFromList(0,FindGraphsWithWave(image))
	if (strlen(win))													// image is already on graph, just bring graph to front
		DoWindow/F $win
		return win
	endif
	Variable faint=55000												// or try 4500

	String wnote=note(image), title, str
	Variable scanNum=NumberByKey("scanNum",wnote,"="), point=NumberByKey("point",wnote,"=")
	Variable keV=specInfo(scanNum,"DCM_energy")
	Variable filters=NumberByKey("filters",wnote,"="), trans=NumberByKey("trans",wnote,"=")
	Variable Nx=DimSize(image,0), Ny=DimSize(image,1)
	Variable dMarker=round(max(Nx,Ny)/10)
	dMarker = min(dMarker,min(Nx,Ny))
	String comment = specInfoT(scanNum,"comment")
	String timeWritten = specInfoT(scanNum,"timeWritten")
	String command = specInfoT(scanNum,"specCommand")
	String fileName=StringByKey("file_name",wnote,"=")
	sprintf title,"scan%d - #%d,   %s",scanNum,point,comment
	if (strlen(timeWritten))
		title += "\r"+specTime2Igor(timeWritten,2)
	endif
	if (numtype(filters+trans)==0 && filters>0)
		sprintf str, "filter[%s] = %.5g",specProc#filterNumber2Name(filters),trans
		title += "\r"+str
	endif

	String DiffractometerAxisNames = StrVarOrDefault("root:Packages:Diffractometer:DiffractometerAxisNames","")
	Variable Naxes = ItemsInList(DiffractometerAxisNames)
	if (Naxes>0)														// add the diffractometer angles
		String aName, str1
		Variable i, angle
		str = ""
		for (i=0;i<Naxes;i+=1)
			aName = StringFromList(i,DiffractometerAxisNames)
			angle = NumberByKey(aName,wnote,"=")					// first check wavenote for angle
			angle = numtype(angle) ? specInfo(scanNum,aName) : angle	// angle is not in wavenote, so check specInfo()
			sprintf str1, "%s = %.3f",aName,angle
			str += SelectString(i>0,"",", ")+str1						// accumulate the "name=angle, "
		endfor
		title += "\r\\Zr070"+str+"\\M"
	endif
	if (strlen(command))
		title += "\r\\Zr075"+command+"\\M"
	endif
	if (strlen(fileName))
		title += "\r\\Zr075"+fileName+"\\M"
	endif

	Display /W=(265,50,1001,350)									// build a new graph
	AppendImage image
	ModifyImage $NameOfWave(image) ctab= {*,*,Terrain256,1}
	ModifyGraph grid=1, tick=2, mirror=1, minor=1, lowTrip=0.001, axOffset(left)=-1.1,axOffset(bottom)=-1
	ModifyGraph gridRGB=(faint,faint,65535), gridStyle=1
	Label left "y  (\\U)"
	Label bottom "x  (\\U)"
	SetAxis/A/R left
	TextBox/C/N=textTitle/F=0/B=1/A=LB/X=2/Y=4 title
	DrawMarker((Nx-1)/2,(Ny-1)/2,dMarker,dMarker,"cross gap",dash=2,layer="UserAxes")
	SetWindow kwTopWin,hook(peakInfo)=specImage#getQofImageHook
	DoUpdate
	win = StringFromList(0,WinList("*",";","WIN:"))
	SetAspectToSquarePixels(win)
	return win
End
//
//		SetWindow kwTopWin,hook(peakInfo)=getQofImageHook
Static Function getQofImageHook(s)										// Shift-mouseDown=follow mouse and show Qvector
	STRUCT WMWinHookStruct &s
	String win = (s.winName)

	if (s.eventCode==5)												// this lets you leave the window up temporaily
		return 0
	elseif (s.eventMod!=3)
		Tag/K/N=indexedPeakInfo/W=$win
		s.doSetCursor= 0
		s.cursorCode = 0											// for nothing
		return 0
	endif

	String imageNameStr=StringFromList(0,ImageNameList(win,";"))
	Wave image = ImageNameToWaveRef(win,imageNameStr)
	if (!WaveExists(image))
		return 1
	endif

	STRUCT detectorGeometry d
	FillDetectorStructDefault(d)									//fill the geometry structure with current values
	String wnote=note(image)

	Variable Naxes=NumVarOrDefault("root:Packages:Diffractometer:Naxes",NaN)
	String DiffractometerAxisNames = StrVarOrDefault("root:Packages:Diffractometer:DiffractometerAxisNames","")
	if (numtype(Naxes)||strlen(DiffractometerAxisNames)<1 || Naxes<1)
		return 1
	endif
	Variable scanNum=NumberByKey("scanNum",wnote,"=")
	Variable keV=specInfo(scanNum,"DCM_energy")
	Make/N=(Naxes)/D/FREE A=NumberByKey(StringFromLIst(p,DiffractometerAxisNames),wnote,"=")
	A = numtype(A[p]) ? specInfo(scanNum,StringFromLIst(p,DiffractometerAxisNames)) : A[p]	// angle is not in wavenote, so check specInfo()
	if (numtype(scanNum+keV+sum(A)))
		return 1
	endif

	GetWindow $win psize
	Variable vert = limit((s.mouseLoc.v-V_top)/(V_bottom-V_top),0,1)	// fractional position on graph
	Variable horiz = limit((s.mouseLoc.h-V_left)/(V_right-V_left),0,1)
	Variable theta, px,py											// pixel position at the mouse-click
	GetAxis/W=$win/Q bottom
	if (V_flag)
		return 1
	endif
	px = (V_max-V_min)*horiz + V_min
	GetAxis/W=$win/Q left
	if (V_flag)
		return 1
	endif
	py = (V_min-V_max)*vert + V_max

	Make/N=3/D/FREE qvec=NaN
	theta = diffractometer#QofPixel(d,keV,px,py,A,qvec)			// returns Q of pixel in SAMPLE (not xtal or beam line) frame

	STRUCT sampleStructure sa	
	String str = ParseFilePath(1,GetWavesDataFolder(image,1),":",1,0)+"sampleStructStr"
	String strStruct=StrVarOrDefault(str,"")						// fill the sample structure with values in spec scan directory
	if (strlen(strStruct)<1)
		return 1
	endif
	StructGet/S/B=2 sa, strStruct									// found structure information, load into s
	Wave hkl = HKLofPixel(sa,d,keV,px,py,A)						// returns hkl of pixel in beam CRYSTAL frame, (not sample frame, not BL coords)

	String tagStr=""
	sprintf str,"\f01q\f00\\BBL\\M = {%.3f, %.3f, %.3f} nm\\S-1\\M |q|= %.3f",qvec[0],qvec[1],qvec[2],norm(qvec)
	tagStr += str
	if (WaveExists(hkl))
		sprintf str,"\r\f01hkl\f00 = [%.3f, %.3f, %.3f]",hkl[0],hkl[1],hkl[2]
		tagStr += str
	endif
	sprintf str,"\r\\[0pixel [%.2f, %.2f],  \\F'Symbol'\\Zr1002q\\]0 = %.4f\\F'Symbol'\\]0",px,py,2*theta*180/PI
	tagStr += str

	px = limit(px,0,DimSize(image,0)-1)							// needed in case (px,py) is outside the image
	py = limit(py,0,DimSize(image,1)-1)
	Variable index = round(px) + DimSize(image,0)*round(py)		// convert px,py back to index into image, for positioning text box
	GetAxis/W=$win/Q bottom
	horiz = (px-V_min)/ (V_max-V_min)							// determine anchor for text box
	GetAxis/W=$win/Q left
	vert = (py-V_max)/(V_min-V_max)
	Variable x0 = horiz<0.5 ? 5 : -5,  y0 = vert<0.5 ? -5 : 5
	String anchor = SelectString(horiz<0.5,"R","L")+ SelectString(vert<0.5,"B","T")
	Tag/C/N=indexedPeakInfo/W=$win/A=$anchor/F=2/L=2/X=(x0)/Y=(y0)/P=1 $imageNameStr,index,tagStr
	s.doSetCursor= 1
	s.cursorCode = 21												// point marker
	DoUpdate
	return 1														// 1 means do not send back to Igor for more processing
End

//  ============================ End of spec Image Plotting =============================  //
//  ============================================================================  //
