#pragma rtGlobals=1		// Use modern global access method.
#pragma IgorVersion = 4.0
#pragma version = 3.0



Function KB_mirror_InitPackage()
	Silent 1
	NewDataFolder /O root:Packages
	NewDataFolder /O root:Packages:KBmirror
	if (exists("root:Packages:KBmirror:rangeL0")!=2)		// point to start fitting on long mirror
		Variable /G root:Packages:KBmirror:rangeL0=-inf
	endif
	if (exists("root:Packages:KBmirror:rangeL1")!=2)		// point to end fitting on long mirror
		Variable /G root:Packages:KBmirror:rangeL1=inf
	endif
	if (exists("root:Packages:KBmirror:rangeS0")!=2)		// point to start fitting on short mirror
		Variable /G root:Packages:KBmirror:rangeS0=-inf
	endif
	if (exists("root:Packages:KBmirror:rangeS1")!=2)		// point to end fitting on short mirror
		Variable /G root:Packages:KBmirror:rangeS1=inf
	endif
	if (exists("root:Packages:KBmirror:rangeX0")!=2)		// point to start fitting on arbitrary mirror
		Variable /G root:Packages:KBmirror:rangex0=-inf
	endif
	if (exists("root:Packages:KBmirror:rangeX1")!=2)		// point to end fitting on arbitrary mirror
		Variable /G root:Packages:KBmirror:rangeX1=inf
	endif
	if (exists("root:Packages:KBmirror:last_s1")!=2)		// last s1 used
		Variable /G root:Packages:KBmirror:last_s1=60
	endif
	if (exists("root:Packages:KBmirror:last_s2")!=2)		// last s2 used
		Variable /G root:Packages:KBmirror:last_s2=.100
	endif
	if (exists("root:Packages:KBmirror:fit_s2")!=2)		// default value for whether to fit s2 (1=fit, 2=constant)
		Variable /G root:Packages:KBmirror:fit_s2=3
	endif
	if (exists("root:Packages:KBmirror:debugLevel")!=2)	// default for debugging (0=none)
		Variable /G root:Packages:KBmirror:debugLevel=0
	endif

//	if (exists("root:Packages:KBmirror:xoff")!=2)			// default value for xoff (1=fit, 2=constant)
//		Variable /G root:Packages:KBmirror:xoff=1
//	endif

//	if (exists("root:Packages:KBmirror:KBmirrorDefaultFile")!=2)
//		// name of last KBmirror data file read
//		String /G root:Packages:KBmirror:KBmirrorDefaultFile=""
//	endif
//	if (exists("root:Packages:KBmirror:KBmirrorDefaultPath")!=2)
//		// operating system path where KBmirror data files exist for reading
//		String /G root:Packages:KBmirror:KBmirrorDefaultPath="home"
//	endif
//	if (exists("root:Packages:KBmirror:KBmirrorDataFolder")!=2)
//		// igor data folder currently used to store KBmirror scans
//		String /G root:Packages:KBmirror:KBmirrorDataFolder="root:raw:"
//	endif
End

Function AppendPlotsToNotebook(graphFit,graphHeight,graphDeposit)
	String graphFit				// name of graph with fit to LTP
	String graphHeight			// name of graph showing  height from LTP
	String graphDeposit			// name of graph showing deposition profile

	if (strlen(graphFit)<1 || strlen(graphHeight)<1 || strlen(graphDeposit)<1)
		String fitList = CheckWinListForNotes(WinList("*", ";", "WIN:1"),"fitLTP")
		String heightList = CheckWinListForNotes(WinList("*", ";", "WIN:1"),"height")
		String DepositList = CheckWinListForNotes(WinList("*", ";", "WIN:1"),"deposition")
		if (strlen(graphFit)<1)
			graphFit = StringFromList(0, fitList )
		endif
		if (strlen(graphHeight)<1)
			graphHeight = StringFromList(0, heightList )
		endif
		if (strlen(graphDeposit)<1)
			graphDeposit = StringFromList(0, DepositList )
		endif
		Prompt graphFit, "graph with fit to LTP", popup,  "none;"+fitList
		Prompt graphHeight, "graph with height from LTP", popup,  "none;"+heightList
		Prompt graphDeposit, "graph with deposition profile", popup,  "none;"+DepositList

		DoPrompt "select graphs for notebook", graphFit,graphHeight,graphDeposit
		if (V_flag)
			return 1
		endif
	endif

	PauseUpdate
	String nb="KB_Mirror_Results"

	if (strlen(WinList(nb, ";", "WIN:16")) != (strlen(nb)+1))	// need to create notebook
		NewNotebook /F=1 /N=$nb /V=0/K=3
		Notebook $nb statusWidth=238, pageMargins={72,72,72,72}, showRuler=0
		Notebook $nb newRuler=Normal, justification=0, margins={0,0,468}, spacing={0,0,0}, rulerDefaults={"Helvetica",12,0,(0,0,0)}
		Notebook $nb, text="Mirrors from file \""+igorinfo(1)+"\",   "
		Notebook $nb, specialChar={3,0,"" }, text=",   ", specialChar={5,0,"" },text="\r\r"

		Notebook $nb,text="Note, the quoted FWHM will be zero if the fit is perfect "
		Notebook $nb,text="(This implies that a cylinder will focus perfectly).  "
		Notebook $nb,text="The FWHM is just intended to show the significance of "
		Notebook $nb,text="the slope errors.\r"
	else
		Notebook KB_Mirror_Results, selection={endOfFile, endOfFile }	// ensure at EOF
		Notebook $nb, specialChar={1,0,"" }		// insert page break
	endif

	Notebook KB_Mirror_Results, selection={endOfFile, endOfFile }	// force position at EOF

	if (strlen(graphFit)>0 || stringmatch(graphFit,"none"))
		Notebook $nb, frame=0, scaling={50,50}, picture={$graphFit,-1,1}
	endif
	if (strlen(graphHeight)>0 && !stringmatch(graphHeight,"none"))
		Notebook $nb, frame=0, scaling={50,50}, picture={$graphHeight,-1,1}
	endif
	if (strlen(graphDeposit)>0 && !stringmatch(graphDeposit,"none"))
		Notebook $nb, frame=0, picture={$graphDeposit,-1,1}
	endif
End


Function/T CheckWinListForNotes(list,search)
	// search windows for those with 'search' in the window note
	String list				// list of window names
	String search			// string to find in window note (set with SetWindow)

	String win, wnote, listOut=""
	Variable i=0

	do
		win = StringFromList(i,list)
		if (strlen(win)<1)
			break
		endif
		GetWindow $win note
		if (!strsearch(S_value,search,0))
			listOut = AddListItem(win,listOut)
		endif
		i += 1
	while (strlen(win)>0)
//	SetWindow kwTopWin note="this is the note"
	return listOut
End



Function/T CheckWaveListForValues(list,key,lo,hi)
	// search list for those waves with 'key=val' in the wave note, include names with lovalhi
	String list				// list of window names
	String key				// keyword to search for
	Variable lo,hi			// allowed range of values for inclusion

	String wav, listOut=""
	Variable val,i=0

	do
		wav = StringFromList(i,list)
		if (strlen(wav)<1)
			break
		endif
		val = NumberByKey(key,note($wav),"=")
		if (lo<=val && val <= hi)
			listOut = AddListItem(wav,listOut)
		endif
		i += 1
	while (strlen(wav)>0)
	return listOut
End



Function FWHM_of_errorWave(err)
	Wave err							// wave containing spot errors (or any other kind), scaled
	switch(1)
		case 1:
			return FWHM_of_errorWaveSortGaussian(err)
		case 2:
			return FWHM_of_errorWaveGaussian(err)
		case 3:
			return FWHM_of_errorWaveFunky(err)
	endswitch
End
Function FWHM_of_errorWaveFunky(err)	// return fwhm of the spot errors, with wierd integral
	Wave err							// wave containing spot errors (or any other kind), scaled
	Variable fw

	duplicate/O err err__
	sort err__,err__
	wavestats/Q err__
	redimension/N=(V_npnts) err__
	duplicate /O err__ ierr__

	SetScale/P x 0,1,"", ierr__
	integrate ierr__
	ierr__ *= -1
	wavestats/Q ierr__
	FindLevels/Q  ierr__, V_max/2
	Wave W_FindLevels=W_FindLevels

	fw = err__[W_FindLevels[1]]-err__[W_FindLevels[0]]
	KillWaves/Z err__,ierr__,W_FindLevels
	return (fw)
End
Function FWHM_of_errorWaveSortGaussian(err)	// fwhm of the spot errors, using sort and Gaussian areas
	Wave err							// wave containing spot errors (or any other kind), scaled

	duplicate/O err err__
	sort err__,err__
	wavestats/Q err__
	redimension/N=(V_npnts) err__

	Variable xlo,xhi, hi
	hi = V_npnts-1
	xlo = err__[hi*0.119516]			// values chosen for areas of Gaussian distribution
	xhi = err__[hi*0.880484]
	KillWaves/Z err__
	return abs(xhi-xlo)
End
Function FWHM_of_errorWaveGaussian(err)	// fwhm of the spot errors, assumes Gaussian distn
	Wave err							// wave containing spot errors (or any other kind), scaled

	WaveStats/Q err
	Make/N=(V_npnts)/O hist___				// temporary wave for summing
	SetScale/I x (V_min),(V_max),"", hist___

	Histogram/B=2 err,hist___
	SetScale/P x 0,1,"", hist___
	integrate hist___
	SetScale/I x (V_min),(V_max),"", hist___
	Variable xlo, xhi, deltaInt, minInt=hist___[0]
	deltaInt = hist___[inf]-hist___[0]
	FindLevel/Q  hist___, deltaInt*0.119516 + minInt	// was 0.25, new levels chosen assuming gaussian
	xlo = V_LevelX
	FindLevel/Q  hist___, deltaInt*0.880484 + minInt	// was 0.75
	xhi = V_LevelX
	KillWaves/Z hist___
	return (xhi-xlo)
End


Function PutUpGraph_GetRange(range0,range1,rad,radX,windowName)
	Variable &range0,&range1			// default range settings
	Wave rad
	Wave radX
	String windowName
	windowName = UniqueName("range_to_fit",6,0)
	Display /W=(5,42,536,585) rad vs radX
	DoWindow /C $windowName
	PauseUpdate
	ModifyGraph/Z tick=2, minor=1, lowTrip(left)=0.1, standoff=0, mirror=1, gfSize=14
	ModifyGraph/Z mode[0]=3, marker[0]=8, opaque[0]=1
	ModifyGraph/Z rgb[0]=(0,43520,65280), lsize=2
	Label bottom "distance  (\\U)"
	Cursor/P A $NameOfWave(rad) (range0);Cursor/P B $NameOfWave(rad) (range1)
	ShowInfo

	Variable rval= UserCursorAdjust(WinName(0,1))
	if (rval == -1)			// Graph name error?
		return -1
	endif
	if (rval == 1)			// User canceled?
		DoAlert 0,"Canceled"
		return -1
	endif
	range0 = pcsr(A)
	range1 = pcsr(B)
	DoWindow /K $WinName(0,1)						// Kill the window, all down with it
//	printf "range =[%d,%d]\r",range0,range1
End


Static Function UserCursorAdjust(grfName)
	String grfName

	DoWindow/F $grfName		// Bring graph to front
	if (V_Flag == 0)			// Verify that graph exists
		Abort "UserCursorAdjust: No such graph."
		return -1
	endif

	NewDataFolder/O root:tmp_PauseforCursorDF
	Variable/G root:tmp_PauseforCursorDF:canceled= 0
	NewPanel/K=2 /W=(139,341,382,450) as "Pause for Cursor"
	DoWindow/C tmp_PauseforCursor			// Set to an unlikely name
	AutoPositionWindow/E/M=1/R=$grfName	// Put panel near the graph
	DrawText 21,20,"Adjust the cursors to fit range and then"
	DrawText 21,40,"press Continue."
	Button button0,pos={80,58},size={92,20},title="Continue"
	Button button0,proc=UserCursorAdjust_ContButtonProc
	Button button1,pos={80,80},size={92,20}
	Button button1,proc=UserCursorAdjust_CancelBProc,title="Cancel"
	PauseForUser tmp_PauseforCursor,$grfName

	NVAR gCaneled= root:tmp_PauseforCursorDF:canceled
	Variable canceled= gCaneled			// Copy from global to local before global is killed
	KillDataFolder root:tmp_PauseforCursorDF

	return canceled
End
Function UserCursorAdjust_ContButtonProc(ctrlName) : ButtonControl
	String ctrlName
	DoWindow/K tmp_PauseforCursor	// Kill self
End
Function UserCursorAdjust_CancelBProc(ctrlName) : ButtonControl
	String ctrlName
	Variable/G root:tmp_PauseforCursorDF:canceled= 1
	DoWindow/K tmp_PauseforCursor	// Kill self
End



Function/T MakePromptString(val,list)	
// this makes a string for a Prompt for floats that use a popup menu, see example at end
// make a popup menu for floating point numbers
//	evaluate result with 
//	str2num(StringFromList(val-1,list))
	Variable val
	String list

	Variable i, index
	Variable N=ItemsInList(list)
	String sval=num2str(val)

	// check if val is in list, and add to list if it is not
	index = -1
	do
		if (val==str2num(StringFromList(i,list)))
			index = i
			break
		endif
		i += 1
	while(i<N)
	if (index<0)
		list = AddListItem(sval,list,";",0)
	else
		sval = StringFromList(i,list)
		list = RemoveListItem(index,list)	// make sure that current value is first.
		list = AddListItem(sval,list,";",0)
	endif
	return list
	// example
	//	String list=MakePromptString(a,"64.5;36.5")
	//	Prompt a, "value of a", popup, list
	//	a = 0
	//	DoPrompt "test", a
	//	a = str2num(StringFromList(a-1,list))
End




Function goodRMS(ww)
	Wave ww
	Variable n = numpnts(ww)
	Variable j=0,i=0,ss=0,yy
	do
		yy = ww[i]
		if ((yy<0)||(yy>0))
			ss += yy^2
			j += 1
		else
			ww[i] = NaN
		endif
		i += 1
	while (i<n)
	return sqrt(ss/j)
End

Function goodRMS_range(ww,p1,p2)
	Wave ww
	Variable p1,p2

	Variable avg = faverage(ww,pnt2x(ww,p1),pnt2x(ww,p2))

	Variable j=0,i=p1,ss=0,yy
	do
		yy = ww[i]
		if ((yy<0)||(yy>0))
			ss += yy^2
			j += 1
		else
			ww[i] = NaN
		endif
		i += 1
	while (i<=p2)
	return sqrt(ss/j)
End




Function LengthScalng2meters(units)
	String units
	units = ZapControlCodes(units)

	if (cmpstr(units,"micron")==0 || cmpstr(units,"micron")==0)
		return 1e-6	
	endif

	String str = LowerStr(units)
	Variable i = strsearch(str,"meter",0)
	if (i<0)
		i = strsearch(str,"m",strlen(str)-1)		// check if it ends in m
	endif

	if (i<0)
		return NaN
	elseif (i==0)
		return 1
	endif
	return ScalingPrefix(units[0,i-1])
End

Function AngleScalng2radians(units)
	String units

	units = ZapControlCodes(units)
	String str = LowerStr(units)

	Variable i = strsearch(str,"rad",0)
	if (i<0)
		return NaN
	elseif (i==0)
		return 1
	endif
	return ScalingPrefix(units[0,i-1])
End

Function ScalingPrefix(prefix)
	String prefix
	Variable scaling

	if (strlen(prefix)<1)
		return 1
	endif
	if (cmpstr(prefix,"")==0)
		return 1e-6
	endif

	if (strsearch(prefix,UpperStr(prefix[0,0]),0)==0)	// an upper case string
		strswitch(prefix)
			case "T":
				scaling = 1e12
				break
			case "G":
				scaling = 1e9
				break
			case "M":
				scaling = 1e6
				break
			case "K":
				scaling = 1e3
				break
			default:
				scaling = NaN
		endswitch
	else
		strswitch(prefix)
			case "k":
				scaling = 1e3
				break
			case "m":
			case "milli":
				scaling = 1e-3
				break
			case "":
			case "micro":
				scaling = 1e-6
				break
			case "nano":
			case "n":
			case "m":
				scaling = 1e-9
				break
			case "pico":
			case "p":
				scaling = 1e-12
				break
			case "femto":
			case "f":
				scaling = 1e-15
				break
			default:
				scaling = NaN
		endswitch
	endif

	return scaling
End





Function/T ZapControlCodes(str)
	String str
	Variable i = 0
	do
		if (char2num(str[i,i])<32 && char2num(str[i,i])>=-1)
			str[i,i+1] = str[i+1,i+1]
		endif
		i += 1
	while(i<strlen(str))
	return str
End
